// ローンシミュレーション（借入可能金額算出） ver1.0
// 
// E-Mail: info@sitetool.biz
// Web   : http://www.sitetool.biz
// (C) 不動産WEBサイトツール
// 
// 上記著作権表示部は改変不可です。
// その他スクリプト部分は改変可です。

var nenshuu = Class.create({
    initialize: function(elm) {
        
        // エラー格納用変数
        this.errors = [];
        this.error_elm = '';
        
        // 出力エリアのID
        this.elm = elm;
        
        // 計算結果格納用
        this.results = {};
        
        // 入力フォームを出力
        this.putForm();
        // イベントリスナ登録
        this.setEventHandlers();
    },
    // ボタンのイベントリスナ
    setEventHandlers: function() {
        this.clearHandler = this.clearForm.bind(this);
        $('fwt_clear').observe('click', this.clearHandler);
        this.calcHandler = this.calcForm.bind(this);
        $('fwt_calc').observe('click', this.calcHandler);
    },
    // 各種計算と結果出力
    calcForm: function() {
        this.clearResult();
        this.calcBegin();
        this.checkForm();
        if(this.errors.length > 0) {
            this.alerts();
        }
        else {
            // 各値をキャスト
            var values = {};
            values['fwt_income'] = parseFloat($('fwt_income').value);
            values['fwt_income_joint'] = parseFloat($('fwt_income_joint').value);
            values['fwt_rate_income'] = parseFloat($('fwt_rate_income').value/100);
            values['fwt_debt_rate'] = parseFloat($('fwt_debt_rate').value/100);
            values['fwt_pay_year'] = parseInt($('fwt_pay_year').value);
            
            if($('fwt_pay_type2').checked == true) {
                this.results = this.calcGankinKintou(values);
                // 結果出力
                this.putResults();
            }
            else {
                this.results = this.calcGanriKintou(values);
                // 結果出力
                this.putResults();
            }
            
        }
    },
    // 元金均等
    calcGankinKintou: function(values) {
        // 計算例（年）-- 初年度返済額100万円，利率年5.0%，返済期間20年の場合の借入可能額 
        // 1000000*20/(1+5/100*20)  = 10,000,000
        
        // 年間の返済額
        values['fwt_pay_money'] = (values['fwt_income'] + values['fwt_income_joint']) * values['fwt_rate_income'] * 10000;
        // 月の返済額
        values['fwt_pay_money_m'] = values['fwt_pay_money'] / 12;
        
        // 利率（月利）
        values['fwt_rate_m'] = values['fwt_debt_rate'] / 12;
        
        // 返済回数（月）
        values['fwt_pay_m'] = values['fwt_pay_year'] * 12;
        
        // 借入可能額
        values['fwt_debt_total'] = Math.round((values['fwt_pay_money_m'] * values['fwt_pay_m']) / (1+values['fwt_rate_m']*values['fwt_pay_m']));
        
        return values;
    },
    // 元利均等
    calcGanriKintou: function(values) {
        // 計算例（年）-- 年間返済可能額150万円，利率年4.0%，返済期間25年の場合の借入可能金額
        // 1500000*((1+4/100)^25-1)/(4/100*(1+4/100)^25) = 23,433,119 
        
        // 年間の返済額
        values['fwt_pay_money'] = (values['fwt_income'] + values['fwt_income_joint']) * values['fwt_rate_income'] * 10000;
        // 月の返済額
        values['fwt_pay_money_m'] = values['fwt_pay_money'] / 12;
        
        // 利率（月利）
        values['fwt_rate_m'] = values['fwt_debt_rate'] / 12;
        
        // 返済回数（月）
        values['fwt_pay_m'] = values['fwt_pay_year'] * 12;
        
        // 借入可能額
        values['fwt_debt_total'] = Math.round((values['fwt_pay_money_m'] * (Math.pow((1+values['fwt_rate_m']),values['fwt_pay_m']) - 1)) / (values['fwt_rate_m'] * Math.pow((1+values['fwt_rate_m']),values['fwt_pay_m'])));
        
        return values;
    },
    // 入力フォーム出力
    putForm: function() {
        var html = '';
        html += '<p id="fwt_caption_input">年収から借入可能金額をシミュレーションします</p>';
        html += '<div id="fwt_explane">';
        html += '<p>※下記フォームを入力し、「計算」ボタンをクリックしてください。</p>';
        html += '<p>※計算結果は概算です。あくまで参考としてご利用ください。</p>';
        html += '</div>';
        html += '<form name="fwt_nenshuuForm" id="fwt_nenshuuForm">';
        html += '<div id="fwt_inputs">';
        html += '<table>';
        html += '<tr>';
        html += '<th>あなたの年収<span class="fwt_need">必須</span></th>';
        html += '<td><input type="text" name="fwt_income" id="fwt_income" value="" size="10" /> 万円<div class="fwt_note">1～999999</div></td>';
        html += '</tr>';
        html += '<tr>';
        html += '<th>連帯債務者の年収</th>';
        html += '<td><input type="text" name="fwt_income_joint" id="fwt_income_joint" value="" size="10" /> 万円<div class="fwt_note">1～999999</div><div class="fwt_note">※共同で住宅ローンの返済義務を負う方の年収</div></td>';
        html += '</tr>';
        html += '<tr>';
        html += '<th>返済負担率<span class="fwt_need">必須</span></th>';
        html += '<td><input type="text" name="fwt_rate_income" id="fwt_rate_income" value="" size="10" /> ％<div class="fwt_note">0.01～99.99</div><div class="fwt_note">※年収に占める返済額の割合</div></td>';
        html += '</tr>';
        html += '<tr>';
        html += '<th>借入金の予定利率<span class="fwt_need">必須</span></th>';
        html += '<td><input type="text" name="fwt_debt_rate" id="fwt_debt_rate" value="" size="10" /> ％<div class="fwt_note">0.01～99.99</div></td>';
        html += '</tr>';
        html += '<tr>';
        html += '<th>返済期間<span class="fwt_need">必須</span></th>';
        html += '<td>'+this.makePayYear('fwt_pay_year')+'年</td>';
        html += '</tr>';
        html += '<tr>';
        html += '<th>返済方式</th>';
        html += '<td><input type="radio" name="fwt_pay_type" id="fwt_pay_type1" value="1" checked="checked" /><label for="fwt_pay_type1"> 元利均等</label><input type="radio" name="fwt_pay_type" id="fwt_pay_type2" value="2" /><label for="fwt_pay_type2"> 元金均等</label></td>';
        html += '</tr>';
        html += '<tr>';
        html += '<td colspan="2"><div id="fwt_buttons"><input type="button" name="fwt_clear" id="fwt_clear" value="クリア" /> <input type="button" name="fwt_calc" id="fwt_calc" value="計算" /></div></td>';
        html += '</tr>';
        html += '</table>';
        html += '</div>';
        html += '<div id="fwt_results">';
        html += '</div>';
        html += '</form>';
        Element.update($(this.elm),html);
    },
    // 入力フォームの返済期間選択肢（年）
    makePayYear: function(name) {
        var html = '';
        if(name) {
            html += '<select name="'+name+'" id="'+name+'">';
            for(var i=0; i<=40; i++) {
                if(i == 25) {
                    html += '<option value="'+i+'" selected="selected">'+i+'</option>';
                }
                else {
                    html += '<option value="'+i+'">'+i+'</option>';
                }
            }
            html += '</select>';
        }
        return html;
    },
    // 結果出力
    putResults: function() {
        var html = '';
        html += '<p id="fwt_caption_result">シミュレーション結果</p>';
        html += '<table>';
        html += '<tr>';
        html += '<th>借入金額</th>';
        html += '<td><div id="fwt_debt_total">'+this.addComma(Math.round(this.results['fwt_debt_total'] / 10000))+' 万円</div></td>';
        html += '</tr>';
        html += '<tr>';
        html += '<th>返済金額（合計）</th>';
        html += '<td><div id="fwt_pay_total">'+this.addComma(Math.round(this.results['fwt_pay_money'] * this.results['fwt_pay_year'] / 10000))+' 万円</div></td>';
        html += '</tr>';
        html += '<tr>';
        html += '<th>返済金額（年平均）</th>';
        html += '<td><div id="fwt_pay_money_total">'+this.addComma(Math.round(this.results['fwt_pay_money'] * this.results['fwt_pay_year'] / this.results['fwt_pay_year'] / 1000) / 10)+' 万円</div></td>';
        html += '</tr>';
        html += '<tr>';
        html += '<th>返済金額（月平均）</th>';
        html += '<td><div id="fwt_pay_bonus_total">'+this.addComma(Math.round(this.results['fwt_pay_money'] * this.results['fwt_pay_year'] / this.results['fwt_pay_year'] / 1000 / 12) / 10)+' 万円</div></td>';
        html += '</tr>';
        html += '</table>';
        Element.update($('fwt_results'),html);
    },
    // 結果クリア
    clearResult: function() {
        Element.update($('fwt_results'),'');
        this.results = {};
    },
    // 入力値クリア
    clearForm: function() {
        $('fwt_income').value = '';
        $('fwt_income_joint').value = '';
        $('fwt_rate_income').value = '';
        $('fwt_debt_rate').value = '';
        $('fwt_pay_year').selectedIndex = 25;
        $('fwt_pay_type1').checked = true;
        $('fwt_pay_type2').checked = false;
    },
    // 事前処理（計算・入力値チェックの前）
    calcBegin: function() {
        $('fwt_income').value = this.replaceZennumToHannum($('fwt_income').value);
        $('fwt_income_joint').value = this.replaceZennumToHannum($('fwt_income_joint').value);
        $('fwt_rate_income').value = this.replaceZennumToHannum($('fwt_rate_income').value);
        $('fwt_debt_rate').value = this.replaceZennumToHannum($('fwt_debt_rate').value);
        $('fwt_pay_year').value = this.replaceZennumToHannum($('fwt_pay_year').value);
        if($('fwt_income_joint').value == '') {
            $('fwt_income_joint').value = 0;
        }
    },
    // 入力値チェック
    checkForm: function() {
        this.errors = [];
        this.error_elm = '';
        var name = '';
        
        // あなたの年収
        name = '「あなたの年収」';
        if($('fwt_income').value == '') {
            this.errors.push(name+'を入力してください。');
            this.errorFocus('fwt_income');
        }
        else {
            var err = this.checkNumber($('fwt_income').value,6,0);
            if(err) {
                this.errors.push(name+'は'+err);
                this.errorFocus('fwt_income');
            }
            else {
                if(parseFloat($('fwt_income').value) <= 0) {
                    this.errors.push(name+'は1以上を入力してください。');
                    this.errorFocus('fwt_income');
                }
            }
        }
        
        // 連帯債務者の年収
        name = '「連帯債務者の年収」';
        if($('fwt_income_joint').value != '') {
            var err = this.checkNumber($('fwt_income_joint').value,6,0);
            if(err) {
                this.errors.push(name+'は'+err);
                this.errorFocus('fwt_income_joint');
            }
        }
        
        // 返済負担率
        name = '「返済負担率」';
        if($('fwt_rate_income').value == '') {
            this.errors.push(name+'を入力してください。');
            this.errorFocus('fwt_rate_income');
        }
        else {
            var err = this.checkNumber($('fwt_rate_income').value,2,2);
            if(err) {
                this.errors.push(name+'は'+err);
                this.errorFocus('fwt_rate_income');
            }
            else {
                if(parseFloat($('fwt_rate_income').value) <= 0) {
                    this.errors.push(name+'は0.01以上を入力してください。');
                    this.errorFocus('fwt_rate_income');
                }
            }
        }
        
        // 借入金の予定利率
        name = '「借入金の予定利率」';
        if($('fwt_debt_rate').value == '') {
            this.errors.push(name+'を入力してください。');
            this.errorFocus('fwt_debt_rate');
        }
        else {
            var err = this.checkNumber($('fwt_debt_rate').value,2,2);
            if(err) {
                this.errors.push(name+'は'+err);
                this.errorFocus('fwt_debt_rate');
            }
            else {
                if(parseFloat($('fwt_debt_rate').value) <= 0) {
                    this.errors.push(name+'は0.01以上を入力してください。');
                    this.errorFocus('fwt_debt_rate');
                }
            }
        }
        
        // 返済期間
        name = '「返済期間」';
        if($('fwt_pay_year').value == 0) {
            this.errors.push(name+'を入力してください。');
            this.errorFocus('fwt_pay_year');
        }
        
    },
    // 数値チェック
    checkNumber: function(val,int_length,dec_length) {
        // 空
        if(val == '') {
            return '数値を入力してください。';
        }
        // 数字とカンマ以外
        if(val.match(/[^0-9|^.]/g)) {
            return '数値を入力してください。';
        }
        // カンマの数
        var cnt = 0;
        for(var i=0; i<val.length; i++) {
            if(val.charAt(i) == '.') {
                cnt++;
            }
        }
        if(cnt > 1) {
            return '正しい数値を入力してください。';
        }
        // 先頭カンマ
        if(val.charAt(0) == '.') {
            return '正しい数値を入力してください。';
        }
        // 整数部分の桁数
        if(int_length > 0) {
            var int_str = '';
            if(cnt == 1) {
                int_str = val.substring(0, val.indexOf('.'));
            }
            else {
                int_str = val;
            }
            if(int_str.length > int_length) {
                return '整数部分を'+int_length+'桁以下で入力してください。';
            }
        }
        // 小数部分の桁数
        if(dec_length > 0) {
            var dec_str = '';
            if(cnt == 1) {
                var idx = val.indexOf('.');
                dec_str = val.substring(idx);
                if(dec_str.length-1 > dec_length) {
                    return '小数部分を'+dec_length+'桁以下で入力してください。';
                }
            }
        }
        else {
            if(cnt == 1) {
                return '整数を入力してください。';
            }
        }
        return false;
    },
    // 全角数字＆ドットを半角に置換
    replaceZennumToHannum: function(chars) {
        var zennums = new Array('０','１','２','３','４','５','６','７','８','９','．');
        var hannums = new Array(0,1,2,3,4,5,6,7,8,9,'.');
        for(var i=0; i<11; i++) {
            chars = chars.replace(new RegExp(zennums[i],'g'),hannums[i]); 
        }
        return chars;
    },
    // カンマ付加
    addComma: function(str) {
        var num = new String(str).replace(/,/g, "");
        while(num != (num = num.replace(/^(-?\d+)(\d{3})/, "$1,$2")));
        return num;
    },
    // アラート出力
    alerts: function() {
        if(this.errors.length > 0) {
            var txt = '';
            for(var i=0; i<this.errors.length; i++) {
                txt += this.errors[i]+"\n";
            }
            if(this.error_elm != '') {
                $(this.error_elm).focus();
            }
            alert(txt);
        }
    },
    // エラー要素へのフォーカス
    errorFocus: function(elm) {
        if(this.error_elm == '') {
            this.error_elm = elm;
        }
    }
});

document.observe('dom:loaded', function() {
    
    // 出力スペースの要素名
    var elm = 'fwt_nenshuu';
    
    if($(elm)) {
        new nenshuu(elm);
    }
});


